/*
 * Kerneos
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: jasmine AT ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * $Id$
 */


package org.ow2.kerneos.core.managers {
import com.adobe.cairngorm.Consumer;
import com.adobe.cairngorm.business.ServiceLocator;

import flash.events.Event;
import flash.utils.Dictionary;

import mx.messaging.events.MessageEvent;

import org.ow2.kerneos.common.vo.ModuleFragmentEventVO;
import org.ow2.kerneos.common.vo.ModuleFragmentVO;
import org.ow2.kerneos.common.vo.ModuleVO;
import org.ow2.kerneos.core.api.event.ModuleFragmentEvent;
import org.ow2.kerneos.core.model.KerneosModelLocator;
import org.ow2.kerneos.core.view.fragments.FragmentView;
import org.ow2.kerneos.core.view.window.ModuleWindow;
import org.ow2.kerneos.core.view.window.SwfModuleWindow;

/**
 * @author Loic Albertin
 */
public class ModulesFragmentsLifeCycleManager {

    /**
     * The gravity consumer for asynchronous OSGi communication
     */
    private static var consumer:Consumer = null;

    private static var queuedModuleFragmentsEventsBySwfModuleWindow:Dictionary = new Dictionary();

    private static var viewMap:Object = new Object();


    /**
     * Subscribe a gravity consumer to the kerneos topic
     */
    public static function subscribe():void {
        consumer = ServiceLocator.getInstance().getConsumer("kerneosAsyncModuleFragmentConfigService");
        consumer.addEventListener(MessageEvent.MESSAGE, onModuleEventMessage);
        consumer.subscribe();
    }

    public static function unsubscribe():void {
        consumer.removeEventListener(MessageEvent.MESSAGE, onModuleEventMessage);
        consumer.unsubscribe();
    }

    public static function installModuleFragment(moduleFragment:ModuleFragmentVO):void {
        //Currently we restrict fragments to SwfModule
        var swfModuleWindow:SwfModuleWindow = getSwfModuleWindows(moduleFragment);
        if (swfModuleWindow != null) {
            if (swfModuleWindow.currentState == "loaded") {
                loadFragment(moduleFragment, swfModuleWindow);
            } else if (swfModuleWindow.currentState == "loading") {
                swfModuleWindow.addEventListener("currentStateChange", loadingComplete);
                var moduleFragmentEvent:ModuleFragmentEventVO = new ModuleFragmentEventVO();
                moduleFragmentEvent.moduleFragment = moduleFragment;
                moduleFragmentEvent.eventType = ModuleFragmentEventVO.LOAD;
                if (!queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow]) {
                    queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] = new Array(moduleFragmentEvent);
                } else {
                    (queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] as Array).push(moduleFragmentEvent);
                }
            }
        }
    }


    public static function unloadFragmentsByModule(module:ModuleVO):void {
        if (KerneosModelLocator.getInstance().fragmentsByModules[module.id]) {
            for each (var moduleFragment:ModuleFragmentVO in KerneosModelLocator.getInstance().fragmentsByModules[module.id]) {
                var swfModuleWindow:SwfModuleWindow = getSwfModuleWindows(moduleFragment);
                if (swfModuleWindow != null) {
                    unloadFragment(moduleFragment, swfModuleWindow);
                }
            }
            delete KerneosModelLocator.getInstance().fragmentsByModules[module.id];
        }
    }

    private static function onModuleEventMessage(event:MessageEvent):void {
        if (event.message.body is ModuleFragmentEventVO) {
            var moduleFragmentEvent:ModuleFragmentEventVO = event.message.body as ModuleFragmentEventVO;

            //if the event is a ModuleEvent type and it has a moduleVO
            if (moduleFragmentEvent && moduleFragmentEvent.moduleFragment) {
                var swfModuleWindow:SwfModuleWindow = getSwfModuleWindows(moduleFragmentEvent.moduleFragment);
                if (swfModuleWindow != null) {
                    if (swfModuleWindow.currentState == "loaded") {
                        if (moduleFragmentEvent.eventType == ModuleFragmentEventVO.LOAD) {
                            loadFragment(moduleFragmentEvent.moduleFragment, swfModuleWindow);
                        } else {
                            unloadFragment(moduleFragmentEvent.moduleFragment, swfModuleWindow);
                        }
                    } else if (swfModuleWindow.currentState == "loading") {
                        swfModuleWindow.addEventListener("currentStateChange", loadingComplete);
                        if (!queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow]) {
                            queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] = new Array(moduleFragmentEvent);
                        } else {
                            (queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] as Array).push(moduleFragmentEvent);
                        }
                    }
                }
            }
        }
    }

    private static function getSwfModuleWindows(moduleFragment:ModuleFragmentVO):SwfModuleWindow {
        var moduleId:String = moduleFragment.moduleId;
        var moduleVO:ModuleVO = ModulesLifeCycleManager.getModule(moduleId);
        if (moduleVO != null) {
            var moduleWindow:ModuleWindow = ModulesLifeCycleManager.getModuleWindow(moduleVO);
            if (moduleWindow is SwfModuleWindow) {
                //Currently we restrict fragments to SwfModule
                return moduleWindow as SwfModuleWindow;
            }
        }
        return null;
    }

    private static function loadFragment(moduleFragment:ModuleFragmentVO, swfModuleWindow:SwfModuleWindow):void {
        var fragmentView:FragmentView = new FragmentView();
        fragmentView.moduleFragment = moduleFragment;
        fragmentView.parentModule = swfModuleWindow.module;
        //TODO check if sometimes loading/unloading of the view should be delegated to the Module
        fragmentView.load();
        viewMap[moduleFragment.id] = fragmentView;
        // Dispatch an event to the module in order to let it do something with the view
        swfModuleWindow.getKerneosModule().getDispatcher().dispatchEvent(new ModuleFragmentEvent(ModuleFragmentEvent.FRAGMENT_VIEW_ARRIVAL, fragmentView));
    }

    private static function unloadFragment(moduleFragment:ModuleFragmentVO, swfModuleWindow:SwfModuleWindow):void {
        if (viewMap[moduleFragment.id]) {
            var fragmentView:FragmentView = viewMap[moduleFragment.id];
            //TODO check if sometimes loading/unloading of the view should be delegated to the Module
            fragmentView.unload();
            // Dispatch an event to the module in order to remove any reference to the view
            swfModuleWindow.getKerneosModule().getDispatcher().dispatchEvent(new ModuleFragmentEvent(ModuleFragmentEvent.FRAGMENT_VIEW_DEPARTURE, fragmentView));
            delete viewMap[moduleFragment.id];
        }
    }

    private static function loadingComplete(event:Event):void {
        var swfModuleWindow:SwfModuleWindow = event.currentTarget as SwfModuleWindow;
        //if changing state from "loading" to "loaded" then perform operations
        //otherwise (in case of loading error for example then just remove the listener and queued events
        if (swfModuleWindow.currentState == "loaded") {
            if (queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] &&
                    queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] is Array) {
                while ((queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] as Array).length > 0) {
                    var moduleFragmentEvent:ModuleFragmentEventVO =
                            (queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow] as Array).shift();
                    var moduleFragment:ModuleFragmentVO = moduleFragmentEvent.moduleFragment;
                    if (moduleFragmentEvent.eventType == ModuleFragmentEventVO.LOAD) {
                        loadFragment(moduleFragment, swfModuleWindow);
                    } else {
                        unloadFragment(moduleFragment, swfModuleWindow);
                    }
                }
            }
        }
        if (queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow]) {
            delete queuedModuleFragmentsEventsBySwfModuleWindow[swfModuleWindow];
        }
        swfModuleWindow.removeEventListener("currentStateChange", loadingComplete);
    }


}
}
